@@ -27,42 +27,55 @@
 # See the README file for information on usage and redistribution.
 #
 
+import io
+import struct
+import sys
+import warnings
+
 from . import Image
 from ._util import isPath
-import io
-import os
-import sys
-import struct
 
 MAXBLOCK = 65536
 
-SAFEBLOCK = 1024*1024
+SAFEBLOCK = 1024 * 1024
 
 LOAD_TRUNCATED_IMAGES = False
+"""Whether or not to load truncated image files. User code may change this."""
 
 ERRORS = {
     -1: "image buffer overrun error",
     -2: "decoding error",
     -3: "unknown error",
     -8: "bad configuration",
-    -9: "out of memory error"
+    -9: "out of memory error",
 }
-
-
-def raise_ioerror(error):
-    try:
-        message = Image.core.getcodecstatus(error)
-    except AttributeError:
-        message = ERRORS.get(error)
-    if not message:
-        message = "decoder error %d" % error
-    raise IOError(message + " when reading image file")
+"""Dict of known error codes returned from :meth:`.PyDecoder.decode`."""
 
 
 #
 # --------------------------------------------------------------------
 # Helpers
 
+
+def raise_oserror(error):
+    try:
+        message = Image.core.getcodecstatus(error)
+    except AttributeError:
+        message = ERRORS.get(error)
+    if not message:
+        message = f"decoder error {error}"
+    raise OSError(message + " when reading image file")
+
+
+def raise_ioerror(error):
+    warnings.warn(
+        "raise_ioerror is deprecated and will be removed in a future release. "
+        "Use raise_oserror instead.",
+        DeprecationWarning,
+    )
+    return raise_oserror(error)
+
+
 def _tilesort(t):
     # sort on offset
     return t[2]
@@ -72,15 +85,20 @@ def _tilesort(t):
 # --------------------------------------------------------------------
 # ImageFile base class
 
+
 class ImageFile(Image.Image):
-    "Base class for image file format handlers."
+    """Base class for image file format handlers."""
 
     def __init__(self, fp=None, filename=None):
-        Image.Image.__init__(self)
+        super().__init__()
 
         self._min_frame = 0
 
+        self.custom_mimetype = None
+
         self.tile = None
+        """ A list of tile descriptors, or ``None`` """
+
         self.readonly = 1  # until we know better
 
         self.decoderconfig = ()
@@ -99,27 +117,33 @@ class ImageFile(Image.Image):
             self._exclusive_fp = None
 
         try:
-            self._open()
-        except (IndexError,  # end of data
+            try:
+                self._open()
+            except (
+                IndexError,  # end of data
                 TypeError,  # end of data (ord)
                 KeyError,  # unsupported mode
                 EOFError,  # got header but not the first frame
-                struct.error) as v:
+                struct.error,
+            ) as v:
+                raise SyntaxError(v) from v
+
+            if not self.mode or self.size[0] <= 0:
+                raise SyntaxError("not identified by this driver")
+        except BaseException:
             # close the file only if we have opened it this constructor
             if self._exclusive_fp:
                 self.fp.close()
-            raise SyntaxError(v)
+            raise
 
-        if not self.mode or self.size[0] <= 0:
-            raise SyntaxError("not identified by this driver")
-
-    def draft(self, mode, size):
-        "Set draft mode"
-
-        pass
+    def get_format_mimetype(self):
+        if self.custom_mimetype:
+            return self.custom_mimetype
+        if self.format is not None:
+            return Image.MIME.get(self.format.upper())
 
     def verify(self):
-        "Check file integrity"
+        """Check file integrity"""
 
         # raise exception if something's wrong.  must be called
         # directly after open, and closes file when finished.
@@ -128,19 +152,19 @@ class ImageFile(Image.Image):
         self.fp = None
 
     def load(self):
-        "Load image data based on tile list"
-
-        pixel = Image.Image.load(self)
+        """Load image data based on tile list"""
 
         if self.tile is None:
-            raise IOError("cannot load this image")
+            raise OSError("cannot load this image")
+
+        pixel = Image.Image.load(self)
         if not self.tile:
             return pixel
 
         self.map = None
         use_mmap = self.filename and len(self.tile) == 1
         # As of pypy 2.1.0, memory mapping was failing here.
-        use_mmap = use_mmap and not hasattr(sys, 'pypy_version_info')
+        use_mmap = use_mmap and not hasattr(sys, "pypy_version_info")
 
         readonly = 0
 
@@ -161,8 +185,12 @@ class ImageFile(Image.Image):
         if use_mmap:
             # try memory mapping
             decoder_name, extents, offset, args = self.tile[0]
-            if decoder_name == "raw" and len(args) >= 3 and args[0] == self.mode \
-               and args[0] in Image._MAPMODES:
+            if (
+                decoder_name == "raw"
+                and len(args) >= 3
+                and args[0] == self.mode
+                and args[0] in Image._MAPMODES
+            ):
                 try:
                     if hasattr(Image.core, "map"):
                         # use built-in mapper  WIN32 only
@@ -170,25 +198,28 @@ class ImageFile(Image.Image):
                         self.map.seek(offset)
                         self.im = self.map.readimage(
                             self.mode, self.size, args[1], args[2]
-                            )
+                        )
                     else:
                         # use mmap, if possible
                         import mmap
-                        fp = open(self.filename, "r")
-                        size = os.path.getsize(self.filename)
-                        self.map = mmap.mmap(fp.fileno(), size, access=mmap.ACCESS_READ)
-                        self.im = Image.core.map_buffer(
-                            self.map, self.size, decoder_name, extents, offset, args
+
+                        with open(self.filename) as fp:
+                            self.map = mmap.mmap(
+                                fp.fileno(), 0, access=mmap.ACCESS_READ
                             )
+                        self.im = Image.core.map_buffer(
+                            self.map, self.size, decoder_name, offset, args
+                        )
                     readonly = 1
-                    # After trashing self.im, we might need to reload the palette data.
+                    # After trashing self.im,
+                    # we might need to reload the palette data.
                     if self.palette:
                         self.palette.dirty = 1
-                except (AttributeError, EnvironmentError, ImportError):
+                except (AttributeError, OSError, ImportError):
                     self.map = None
 
         self.load_prepare()
-        err_code = -3 # initialize to unknown error
+        err_code = -3  # initialize to unknown error
         if not self.map:
             # sort tiles in file order
             self.tile.sort(key=_tilesort)
@@ -200,46 +231,44 @@ class ImageFile(Image.Image):
                 prefix = b""
 
             for decoder_name, extents, offset, args in self.tile:
-                decoder = Image._getdecoder(self.mode, decoder_name,
-                                            args, self.decoderconfig)
-                seek(offset)
-                decoder.setimage(self.im, extents)
-                if decoder.pulls_fd:
-                    decoder.setfd(self.fp)
-                    status, err_code = decoder.decode(b"")
-                else:
-                    b = prefix
-                    while True:
-                        try:
-                            s = read(self.decodermaxblock)
-                        except (IndexError, struct.error):  # truncated png/gif
-                            if LOAD_TRUNCATED_IMAGES:
+                decoder = Image._getdecoder(
+                    self.mode, decoder_name, args, self.decoderconfig
+                )
+                try:
+                    seek(offset)
+                    decoder.setimage(self.im, extents)
+                    if decoder.pulls_fd:
+                        decoder.setfd(self.fp)
+                        status, err_code = decoder.decode(b"")
+                    else:
+                        b = prefix
+                        while True:
+                            try:
+                                s = read(self.decodermaxblock)
+                            except (IndexError, struct.error) as e:
+                                # truncated png/gif
+                                if LOAD_TRUNCATED_IMAGES:
+                                    break
+                                else:
+                                    raise OSError("image file is truncated") from e
+
+                            if not s:  # truncated jpeg
+                                if LOAD_TRUNCATED_IMAGES:
+                                    break
+                                else:
+                                    raise OSError(
+                                        "image file is truncated "
+                                        f"({len(b)} bytes not processed)"
+                                    )
+
+                            b = b + s
+                            n, err_code = decoder.decode(b)
+                            if n < 0:
                                 break
-                            else:
-                                raise IOError("image file is truncated")
-
-                        if not s:  # truncated jpeg
-                            self.tile = []
-
-                            # JpegDecode needs to clean things up here either way
-                            # If we don't destroy the decompressor,
-                            # we have a memory leak.
-                            decoder.cleanup()
-
-                            if LOAD_TRUNCATED_IMAGES:
-                                break
-                            else:
-                                raise IOError("image file is truncated "
-                                              "(%d bytes not processed)" % len(b))
-
-                        b = b + s
-                        n, err_code = decoder.decode(b)
-                        if n < 0:
-                            break
-                        b = b[n:]
-
-                # Need to cleanup here to prevent leaks in PyPy
-                decoder.cleanup()
+                            b = b[n:]
+                finally:
+                    # Need to cleanup here to prevent leaks
+                    decoder.cleanup()
 
         self.tile = []
         self.readonly = readonly
@@ -252,14 +281,13 @@ class ImageFile(Image.Image):
 
         if not self.map and not LOAD_TRUNCATED_IMAGES and err_code < 0:
             # still raised if decoder fails to return anything
-            raise_ioerror(err_code)
+            raise_oserror(err_code)
 
         return Image.Image.load(self)
 
     def load_prepare(self):
         # create image memory if necessary
-        if not self.im or\
-           self.im.mode != self.mode or self.im.size != self.size:
+        if not self.im or self.im.mode != self.mode or self.im.size != self.size:
             self.im = Image.core.new(self.mode, self.size)
         # create palette (optional)
         if self.mode == "P":
@@ -278,11 +306,15 @@ class ImageFile(Image.Image):
     #     pass
 
     def _seek_check(self, frame):
-        if (frame < self._min_frame or
+        if (
+            frame < self._min_frame
             # Only check upper limit on frames if additional seek operations
             # are not required to do so
-            (not (hasattr(self, "_n_frames") and self._n_frames is None) and
-             frame >= self.n_frames+self._min_frame)):
+            or (
+                not (hasattr(self, "_n_frames") and self._n_frames is None)
+                and frame >= self.n_frames + self._min_frame
+            )
+        ):
             raise EOFError("attempt to seek outside sequence")
 
         return self.tell() != frame
@@ -297,14 +329,12 @@ class StubImageFile(ImageFile):
     """
 
     def _open(self):
-        raise NotImplementedError(
-            "StubImageFile subclass must implement _open"
-            )
+        raise NotImplementedError("StubImageFile subclass must implement _open")
 
     def load(self):
         loader = self._load()
         if loader is None:
-            raise IOError("cannot find loader for this %s file" % self.format)
+            raise OSError(f"cannot find loader for this {self.format} file")
         image = loader.load(self)
         assert image is not None
         # become the other object (!)
@@ -312,17 +342,16 @@ class StubImageFile(ImageFile):
         self.__dict__ = image.__dict__
 
     def _load(self):
-        "(Hook) Find actual image loader."
-        raise NotImplementedError(
-            "StubImageFile subclass must implement _load"
-            )
+        """(Hook) Find actual image loader."""
+        raise NotImplementedError("StubImageFile subclass must implement _load")
 
 
-class Parser(object):
+class Parser:
     """
     Incremental image parser.  This class implements the standard
     feed/close consumer interface.
     """
+
     incremental = None
     image = None
     data = None
@@ -343,7 +372,7 @@ class Parser(object):
         (Consumer) Feed data to the parser.
 
         :param data: A string buffer.
-        :exception IOError: If the parser failed to parse the image file.
+        :exception OSError: If the parser failed to parse the image file.
         """
         # collect data
 
@@ -375,7 +404,7 @@ class Parser(object):
                 if e < 0:
                     # decoding error
                     self.image = None
-                    raise_ioerror(e)
+                    raise_oserror(e)
                 else:
                     # end of image
                     return
@@ -394,7 +423,7 @@ class Parser(object):
             try:
                 with io.BytesIO(self.data) as fp:
                     im = Image.open(fp)
-            except IOError:
+            except OSError:
                 # traceback.print_exc()
                 pass  # not enough data
             else:
@@ -407,15 +436,13 @@ class Parser(object):
                     im.load_prepare()
                     d, e, o, a = im.tile[0]
                     im.tile = []
-                    self.decoder = Image._getdecoder(
-                        im.mode, d, a, im.decoderconfig
-                        )
+                    self.decoder = Image._getdecoder(im.mode, d, a, im.decoderconfig)
                     self.decoder.setimage(im.im, e)
 
                     # calculate decoder offset
                     self.offset = o
                     if self.offset <= len(self.data):
-                        self.data = self.data[self.offset:]
+                        self.data = self.data[self.offset :]
                         self.offset = 0
 
                 self.image = im
@@ -431,7 +458,7 @@ class Parser(object):
         (Consumer) Close the stream.
 
         :returns: An image object.
-        :exception IOError: If the parser failed to parse the image file either
+        :exception OSError: If the parser failed to parse the image file either
                             because it cannot be identified or cannot be
                             decoded.
         """
@@ -441,9 +468,9 @@ class Parser(object):
             self.feed(b"")
             self.data = self.decoder = None
             if not self.finished:
-                raise IOError("image was incomplete")
+                raise OSError("image was incomplete")
         if not self.image:
-            raise IOError("cannot parse this image")
+            raise OSError("cannot parse this image")
         if self.data:
             # incremental parsing not possible; reopen the file
             # not that we have all data
@@ -457,6 +484,7 @@ class Parser(object):
 
 # --------------------------------------------------------------------
 
+
 def _save(im, fp, tile, bufsize=0):
     """Helper to save image based on tile list
 
@@ -481,12 +509,12 @@ def _save(im, fp, tile, bufsize=0):
     try:
         fh = fp.fileno()
         fp.flush()
-    except (AttributeError, io.UnsupportedOperation):
+    except (AttributeError, io.UnsupportedOperation) as exc:
         # compress to Python file-compatible object
         for e, b, o, a in tile:
             e = Image._getencoder(im.mode, e, a, im.encoderconfig)
             if o > 0:
-                fp.seek(o, 0)
+                fp.seek(o)
             e.setimage(im.im, b)
             if e.pushes_fd:
                 e.setfd(fp)
@@ -498,14 +526,14 @@ def _save(im, fp, tile, bufsize=0):
                     if s:
                         break
             if s < 0:
-                raise IOError("encoder error %d when writing image file" % s)
+                raise OSError(f"encoder error {s} when writing image file") from exc
             e.cleanup()
     else:
         # slight speedup: compress to real file object
         for e, b, o, a in tile:
             e = Image._getencoder(im.mode, e, a, im.encoderconfig)
             if o > 0:
-                fp.seek(o, 0)
+                fp.seek(o)
             e.setimage(im.im, b)
             if e.pushes_fd:
                 e.setfd(fp)
@@ -513,7 +541,7 @@ def _save(im, fp, tile, bufsize=0):
             else:
                 s = e.encode_to_file(fh, bufsize)
             if s < 0:
-                raise IOError("encoder error %d when writing image file" % s)
+                raise OSError(f"encoder error {s} when writing image file")
             e.cleanup()
     if hasattr(fp, "flush"):
         fp.flush()
@@ -543,7 +571,7 @@ def _safe_read(fp, size):
     return b"".join(data)
 
 
-class PyCodecState(object):
+class PyCodecState:
     def __init__(self):
         self.xsize = 0
         self.ysize = 0
@@ -551,14 +579,13 @@ class PyCodecState(object):
         self.yoff = 0
 
     def extents(self):
-        return (self.xoff, self.yoff,
-                self.xoff+self.xsize, self.yoff+self.ysize)
+        return (self.xoff, self.yoff, self.xoff + self.xsize, self.yoff + self.ysize)
 
 
-class PyDecoder(object):
+class PyDecoder:
     """
     Python implementation of a format decoder. Override this class and
-    add the decoding logic in the `decode` method.
+    add the decoding logic in the :meth:`decode` method.
 
     See :ref:`Writing Your Own File Decoder in Python<file-decoders-py>`
     """
@@ -589,10 +616,10 @@ class PyDecoder(object):
         """
         Override to perform the decoding process.
 
-        :param buffer: A bytes object with the data to be decoded.  If `handles_eof`
-             is set, then `buffer` will be empty and `self.fd` will be set.
-        :returns: A tuple of (bytes consumed, errcode). If finished with decoding
-             return <0 for the bytes consumed. Err codes are from `ERRORS`
+        :param buffer: A bytes object with the data to be decoded.
+        :returns: A tuple of ``(bytes consumed, errcode)``.
+            If finished with decoding return <0 for the bytes consumed.
+            Err codes are from :data:`.ImageFile.ERRORS`.
         """
         raise NotImplementedError()
 
@@ -642,8 +669,10 @@ class PyDecoder(object):
         if self.state.xsize <= 0 or self.state.ysize <= 0:
             raise ValueError("Size cannot be negative")
 
-        if (self.state.xsize + self.state.xoff > self.im.size[0] or
-           self.state.ysize + self.state.yoff > self.im.size[1]):
+        if (
+            self.state.xsize + self.state.xoff > self.im.size[0]
+            or self.state.ysize + self.state.yoff > self.im.size[1]
+        ):
             raise ValueError("Tile cannot extend outside image")
 
     def set_as_raw(self, data, rawmode=None):
@@ -651,14 +680,14 @@ class PyDecoder(object):
         Convenience method to set the internal image from a stream of raw data
 
         :param data: Bytes to be set
-        :param rawmode: The rawmode to be used for the decoder. If not specified,
-             it will default to the mode of the image
+        :param rawmode: The rawmode to be used for the decoder.
+            If not specified, it will default to the mode of the image
         :returns: None
         """
 
         if not rawmode:
             rawmode = self.mode
-        d = Image._getdecoder(self.mode, 'raw', (rawmode))
+        d = Image._getdecoder(self.mode, "raw", (rawmode))
         d.setimage(self.im, self.state.extents())
         s = d.decode(data)
 
