@@ -16,10 +16,12 @@
 # See the README file for information on usage and redistribution.
 #
 
-__version__ = "0.4"
+import io
 
 from . import Image, ImageFile, ImagePalette
-from ._binary import i8, i16be as i16, i32be as i32
+from ._binary import i8
+from ._binary import i16be as i16
+from ._binary import i32be as i32
 
 MODES = {
     # (photoshop mode, bits) -> (pil mode, required channels)
@@ -31,13 +33,14 @@ MODES = {
     (4, 8): ("CMYK", 4),
     (7, 8): ("L", 1),  # FIXME: multilayer
     (8, 8): ("L", 1),  # duotone
-    (9, 8): ("LAB", 3)
+    (9, 8): ("LAB", 3),
 }
 
 
 # --------------------------------------------------------------------.
 # read PSD images
 
+
 def _accept(prefix):
     return prefix[:4] == b"8BPS"
 
@@ -45,10 +48,12 @@ def _accept(prefix):
 ##
 # Image plugin for Photoshop images.
 
+
 class PsdImageFile(ImageFile.ImageFile):
 
     format = "PSD"
     format_description = "Adobe Photoshop"
+    _close_exclusive_fp_after_loading = False
 
     def _open(self):
 
@@ -58,7 +63,7 @@ class PsdImageFile(ImageFile.ImageFile):
         # header
 
         s = read(26)
-        if s[:4] != b"8BPS" or i16(s[4:]) != 1:
+        if not _accept(s) or i16(s[4:]) != 1:
             raise SyntaxError("not a PSD file")
 
         psd_bits = i16(s[22:])
@@ -68,10 +73,10 @@ class PsdImageFile(ImageFile.ImageFile):
         mode, channels = MODES[(psd_mode, psd_bits)]
 
         if channels > psd_channels:
-            raise IOError("not enough channels")
+            raise OSError("not enough channels")
 
         self.mode = mode
-        self.size = i32(s[18:]), i32(s[14:])
+        self._size = i32(s[18:]), i32(s[14:])
 
         #
         # color mode data
@@ -92,13 +97,13 @@ class PsdImageFile(ImageFile.ImageFile):
             # load resources
             end = self.fp.tell() + size
             while self.fp.tell() < end:
-                signature = read(4)
+                read(4)  # signature
                 id = i16(read(2))
                 name = read(i8(read(1)))
                 if not (len(name) & 1):
                     read(1)  # padding
                 data = read(i32(read(4)))
-                if (len(data) & 1):
+                if len(data) & 1:
                     read(1)  # padding
                 self.resources.append((id, name, data))
                 if id == 1039:  # ICC profile
@@ -116,6 +121,8 @@ class PsdImageFile(ImageFile.ImageFile):
             if size:
                 self.layers = _layerinfo(self.fp)
             self.fp.seek(end)
+        self.n_frames = len(self.layers)
+        self.is_animated = self.n_frames > 1
 
         #
         # image descriptor
@@ -123,32 +130,24 @@ class PsdImageFile(ImageFile.ImageFile):
         self.tile = _maketile(self.fp, mode, (0, 0) + self.size, channels)
 
         # keep the file open
-        self._fp = self.fp
+        self.__fp = self.fp
         self.frame = 1
         self._min_frame = 1
 
-    @property
-    def n_frames(self):
-        return len(self.layers)
-
-    @property
-    def is_animated(self):
-        return len(self.layers) > 1
-
     def seek(self, layer):
         if not self._seek_check(layer):
             return
 
         # seek to given layer (1..max)
         try:
-            name, mode, bbox, tile = self.layers[layer-1]
+            name, mode, bbox, tile = self.layers[layer - 1]
             self.mode = mode
             self.tile = tile
             self.frame = layer
-            self.fp = self._fp
+            self.fp = self.__fp
             return name, bbox
-        except IndexError:
-            raise EOFError("no such layer")
+        except IndexError as e:
+            raise EOFError("no such layer") from e
 
     def tell(self):
         # return layer number (0=image, 1..max=layers)
@@ -156,13 +155,21 @@ class PsdImageFile(ImageFile.ImageFile):
 
     def load_prepare(self):
         # create image memory if necessary
-        if not self.im or\
-           self.im.mode != self.mode or self.im.size != self.size:
+        if not self.im or self.im.mode != self.mode or self.im.size != self.size:
             self.im = Image.core.fill(self.mode, self.size, 0)
         # create palette (optional)
         if self.mode == "P":
             Image.Image.load(self)
 
+    def _close__fp(self):
+        try:
+            if self.__fp != self.fp:
+                self.__fp.close()
+        except AttributeError:
+            pass
+        finally:
+            self.__fp = None
+
 
 def _layerinfo(file):
     # read layerinfo block
@@ -207,33 +214,31 @@ def _layerinfo(file):
             mode = None  # unknown
 
         # skip over blend flags and extra information
-        filler = read(12)
+        read(12)  # filler
         name = ""
-        size = i32(read(4))
+        size = i32(read(4))  # length of the extra data field
         combined = 0
         if size:
+            data_end = file.tell() + size
+
             length = i32(read(4))
             if length:
-                mask_y = i32(read(4))
-                mask_x = i32(read(4))
-                mask_h = i32(read(4)) - mask_y
-                mask_w = i32(read(4)) - mask_x
-                file.seek(length - 16, 1)
+                file.seek(length - 16, io.SEEK_CUR)
             combined += length + 4
 
             length = i32(read(4))
             if length:
-                file.seek(length, 1)
+                file.seek(length, io.SEEK_CUR)
             combined += length + 4
 
             length = i8(read(1))
             if length:
                 # Don't know the proper encoding,
                 # Latin-1 should be a good guess
-                name = read(length).decode('latin-1', 'replace')
+                name = read(length).decode("latin-1", "replace")
             combined += length + 1
 
-        file.seek(size - combined, 1)
+            file.seek(data_end)
         layers.append((name, mode, (x0, y0, x1, y1)))
 
     # get tiles
@@ -271,7 +276,7 @@ def _maketile(file, mode, bbox, channels):
             if mode == "CMYK":
                 layer += ";I"
             tile.append(("raw", bbox, offset, layer))
-            offset = offset + xsize*ysize
+            offset = offset + xsize * ysize
 
     elif compression == 1:
         #
@@ -284,11 +289,9 @@ def _maketile(file, mode, bbox, channels):
             layer = mode[channel]
             if mode == "CMYK":
                 layer += ";I"
-            tile.append(
-                ("packbits", bbox, offset, layer)
-                )
+            tile.append(("packbits", bbox, offset, layer))
             for y in range(ysize):
-                offset = offset + i16(bytecount[i:i+2])
+                offset = offset + i16(bytecount[i : i + 2])
                 i += 2
 
     file.seek(offset)
@@ -298,6 +301,7 @@ def _maketile(file, mode, bbox, channels):
 
     return tile
 
+
 # --------------------------------------------------------------------
 # registry
 
@@ -305,3 +309,5 @@ def _maketile(file, mode, bbox, channels):
 Image.register_open(PsdImageFile.format, PsdImageFile, _accept)
 
 Image.register_extension(PsdImageFile.format, ".psd")
+
+Image.register_mime(PsdImageFile.format, "image/vnd.adobe.photoshop")
